from django.conf import settings
from django.db import models
from django.db.models import QuerySet
from django.utils import timezone

from .timestamp import TimeStamp


class SoftDeletionQuerySet(QuerySet):
    def delete(self, user=None):
        return super(SoftDeletionQuerySet, self).update(deleted_at=timezone.now(), deleted_by=user)

    def hard_delete(self):
        return super(SoftDeletionQuerySet, self).delete()

    def alive(self):
        return self.filter(deleted_at=None)

    def dead(self):
        return self.exclude(deleted_at=None)


class SoftDeletionManager(models.Manager):
    def __init__(self, *args, **kwargs):
        self.alive_only = kwargs.pop('alive_only', True)
        super(SoftDeletionManager, self).__init__(*args, **kwargs)

    def get_queryset(self):
        if self.alive_only:
            return SoftDeletionQuerySet(self.model).filter(deleted_at=None)
        return SoftDeletionQuerySet(self.model)

    def hard_delete(self):
        return self.get_queryset().hard_delete()


class SoftDeletionModel(TimeStamp):
    deleted_at = models.DateTimeField(blank=True, null=True)
    deleted_by = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        on_delete=models.SET_NULL,
        null=True,
        blank=True,
        related_name='deleted_%(class)ss'
    )
    objects = SoftDeletionManager()
    all_objects = SoftDeletionManager(alive_only=False)

    class Meta:
        abstract = True
        ordering = ['id']

    def save(self, *args, **kwargs):
        user = kwargs.get('user', None)
        if user:
            if self.pk:
                self.updated_by = user
            else:
                self.created_by = user
                self.updated_by = user
            del kwargs['user']
        super(SoftDeletionModel, self).save(*args, **kwargs)

    def delete(self, *args, **kwargs):
        user = kwargs.get('user', None)
        if user:
            self.deleted_by = user
        self.deleted_at = timezone.now()
        self.save()

    def restore(self, *args, **kwargs):
        user = kwargs.get('user', None)
        if user:
            self.updated_at = user
            self.updated_at = timezone.now()
        self.deleted_by = None
        self.deleted_at = None
        self.save()


from django.urls import path

from .views import authorize_google_drive_access, renew_google_drive_access, get_oath_client_secrets

urlpatterns = [
    path('authorize', authorize_google_drive_access),
    path('renew', renew_google_drive_access),
    path('file', get_oath_client_secrets),
]



import os

from pydrive2.auth import GoogleAuth

CLIENT_SECRETS_FILE_PATH = "credentials.json"


def load_credentials_from_env():
    CLIENT_SECRETS_FROM_ENV = os.environ.get('CLIENT_SECRETS_FROM_ENV', 0)
    CLIENT_SECRETS_FILE_CONTENT = os.environ.get('CLIENT_SECRETS_FILE_CONTENT', '')
    if int(CLIENT_SECRETS_FROM_ENV):
        # We are hosted probably in Heroku
        with open(CLIENT_SECRETS_FILE_PATH, 'wt') as json_file:
            json_file.write(CLIENT_SECRETS_FILE_CONTENT)


def initialize_flow():
    gauth = GoogleAuth()
    gauth.GetFlow()
    gauth.flow.params.update({'access_type': 'offline'})
    gauth.flow.params.update({'approval_prompt': 'force'})
    load_credentials_from_env()
    return gauth


def authorize_gd():
    gauth = initialize_flow()
    gauth.LoadCredentialsFile("credentials.json")
    if gauth.credentials is None:
        # Authenticate if they're not there
        gauth.LocalWebserverAuth()
    elif gauth.access_token_expired:
        # Refresh them if expired
        gauth.Refresh()
    else:
        # Initialize the saved creds
        gauth.Authorize()
    # Save the current credentials to a file
    gauth.SaveCredentialsFile(CLIENT_SECRETS_FILE_PATH)
    return gauth


def renew_authorize_gd():
    gauth = initialize_flow()
    gauth.LocalWebserverAuth()
    if gauth.access_token_expired:
        # Refresh them if expired
        gauth.Refresh()
    else:
        # Initialize the saved creds
        gauth.Authorize()
    # Save the current credentials to a file
    gauth.SaveCredentialsFile(CLIENT_SECRETS_FILE_PATH)
    return gauth



from django.conf import settings
from django.db import models


class TimeStamp(models.Model):
    created_at = models.DateTimeField(auto_now_add=True, null=True, blank=True)
    updated_at = models.DateTimeField(auto_now=True, null=True, blank=True)
    created_by = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        on_delete=models.SET_NULL,
        null=True,
        blank=True,
        related_name='created_%(class)ss'
    )
    updated_by = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        on_delete=models.SET_NULL,
        null=True,
        blank=True,
        related_name='updated_%(class)ss'
    )

    class Meta:
        abstract = True
